@@ -15,6 +15,9 @@
 #include "sbutil.h"
 #include "sandbox.h"
 
+/* The C library might have a macro for this. */
+#undef dprintf
+
 static int print_debug = 0;
 #define dprintf(fmt, args...) do { if (print_debug) printf(fmt, ## args); } while (0)
 #define dputs(str) do { if (print_debug) puts(str); } while (0)
@@ -111,7 +114,7 @@ static void print_sandbox_log(char *sandbox_log)
 		return;
 	}
 
-	sb_eerror("--------------------------- ACCESS VIOLATION SUMMARY ---------------------------\n");
+	sb_eerror("----------------------- SANDBOX ACCESS VIOLATION SUMMARY -----------------------\n");
 	sb_eerror("LOG FILE: \"%s\"\n", sandbox_log);
 
 	while (1) {
@@ -128,13 +131,21 @@ static void print_sandbox_log(char *sandbox_log)
 	sb_eerror("--------------------------------------------------------------------------------\n");
 }
 
+static int stop_count = 5;
+
 static void stop(int signum)
 {
 	if (0 == stop_called) {
 		stop_called = signum;
 		sb_warn("caught signal %d in pid %d", signum, getpid());
-	} else
-		sb_warn("signal already caught and busy still cleaning up!");
+	} else if (--stop_count) {
+		sb_warn("Send signal %i more time%s to force SIGKILL",
+			stop_count, stop_count == 1 ? "" : "s");
+	} else {
+		/* This really should kill all children; see usr1_handler. */
+		kill(child_pid, SIGKILL);
+		stop_count = 1;
+	}
 }
 
 static void usr1_handler(int signum, siginfo_t *siginfo, void *ucontext)
@@ -164,7 +175,9 @@ static int spawn_shell(char *argv_bash[], char **env, int debug)
 
 	/* Child's process */
 	if (0 == child_pid) {
-		int ret = execve(argv_bash[0], argv_bash, env);
+		/* Would be nice if execvpe were in POSIX. */
+		environ = env;
+		int ret = execvp(argv_bash[0], argv_bash);
 		sb_pwarn("failed to exec child");
 		_exit(ret);
 	} else if (child_pid < 0) {
@@ -247,29 +260,57 @@ int main(int argc, char **argv)
 		goto oom_error;
 
 	/* Setup bash argv */
-	str_list_add_item_copy(argv_bash, "/bin/bash", oom_error);
-	str_list_add_item_copy(argv_bash, "-rcfile", oom_error);
-	str_list_add_item_copy(argv_bash, sandbox_info.sandbox_rc, oom_error);
-	if (argc >= 2) {
-		int i;
-
-		str_list_add_item_copy(argv_bash, run_str, oom_error);
-		str_list_add_item_copy(argv_bash, argv[1], oom_error);
-		for (i = 2; i < argc; i++) {
-			char *tmp_ptr;
-
-			tmp_ptr = xrealloc(argv_bash[4],
-					   (strlen(argv_bash[4]) +
-					    strlen(argv[i]) + 2) *
-					   sizeof(char));
-			argv_bash[4] = tmp_ptr;
-
-			snprintf(argv_bash[4] + strlen(argv_bash[4]),
-				 strlen(argv[i]) + 2, " %s",
-				 argv[i]);
-		}
+	if (!opt_use_bash && argc == 2) {
+		/* Backwards compatibility hack: if there's only one argument, and it
+		 * appears to be a shell command (not an absolute path to a program),
+		 * then fallback to running through the shell.
+		 */
+		if (access(argv[1], X_OK))
+			opt_use_bash = true;
 	}
 
+	if (opt_use_bash || argc == 1) {
+		str_list_add_item_copy(argv_bash, "/bin/bash", oom_error);
+		str_list_add_item_copy(argv_bash, "-rcfile", oom_error);
+		str_list_add_item_copy(argv_bash, sandbox_info.sandbox_rc, oom_error);
+		if (argc >= 2) {
+			int i;
+			size_t cmdlen;
+
+			str_list_add_item_copy(argv_bash, run_str, oom_error);
+			str_list_add_item_copy(argv_bash, argv[1], oom_error);
+			cmdlen = strlen(argv_bash[4]);
+			for (i = 2; i < argc; i++) {
+				size_t arglen = strlen(argv[i]);
+				argv_bash[4] = xrealloc(argv_bash[4], cmdlen + arglen + 2);
+				argv_bash[4][cmdlen] = ' ';
+				memcpy(argv_bash[4] + cmdlen + 1, argv[i], arglen);
+				cmdlen += arglen + 1;
+				argv_bash[4][cmdlen] = '\0';
+			}
+		}
+	} else {
+		int i;
+		for (i = 1; i < argc; ++i)
+			str_list_add_item_copy(argv_bash, argv[i], oom_error);
+	}
+
+#ifdef HAVE_PRCTL
+	/* Lock down access to elevated privileges.  In practice, this will block
+	 * use of tools like su and sudo, and will allow use of seccomp bpf.
+	 */
+# ifdef PR_SET_NO_NEW_PRIVS
+	if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0) == -1) {
+		/* Ignore EINVAL in case we're on old kernels.  Unfortunately we can't
+		 * differentiate between EINVAL due to unsupported PR_xxx and EINVAL
+		 * due to bad 2nd/3rd/4th/5th args.
+		 */
+		if (errno != EINVAL)
+			sb_eerror("prctl(PR_SET_NO_NEW_PRIVS) failed");
+	}
+# endif
+#endif
+
 	/* Set up the required signal handlers */
 	int sigs[] = { SIGHUP, SIGINT, SIGQUIT, SIGTERM, SIGUSR1, };
 	struct sigaction act_new, act_old[ARRAY_SIZE(sigs)];
@@ -329,7 +370,7 @@ int main(int argc, char **argv)
 		dputs(sandbox_footer);
 
 	/* Do the right thing and pass the signal back up.  See:
-	 * http://www.cons.org/cracauer/sigint.html
+	 * https://www.cons.org/cracauer/sigint.html
 	 */
 	if (stop_called != SIGUSR1 && WIFSIGNALED(shell_exit)) {
 		int signum = WTERMSIG(shell_exit);
